# fields.py
from typing import Any
from django import forms
from django.forms.widgets import Widget
from ..widgets import SelectSearchTextInput
import random
from ..providers import create_model_field_provider_from_model, create_model_provider_from_model
from ..providers import OnlyAuthenticated

class SelectSearchCharField(forms.CharField):
    widget = SelectSearchTextInput

    def __init__(self,field,model=None, permissions=[OnlyAuthenticated], query_function_name=None, min_search_length=1,*args, **kwargs):
        assert query_function_name or model, f"The query function name cannot be None."
        if model:
            self.model = model
            provider = create_model_field_provider_from_model(model, permission_classes=[permissions])
            self.query_function_name = provider.query_function_name
        else:
            self.query_function_name = query_function_name
    
        self.min_search_length = min_search_length
        self.field = field
        super().__init__(*args, **kwargs)

    def widget_attrs(self, widget: Widget) -> Any:
        attrs = super().widget_attrs(widget)            
        attrs.update({'query_function_name': self.query_function_name,
                      'min_search_length': self.min_search_length,
                      "search_field" : self.field,
                      "id": f"search-input-{random.randint(0, 1000000000)}",
                      })
        return attrs




from typing import Any
from django import forms
from django.forms.widgets import Widget
import random

class MainRelatedField():
    def __init__(self, related_search_input, related_field, *args, **kwargs):
        assert related_search_input is not None, f"The related search input cannot be None."
        assert related_field is not None, f"The related field cannot be None."
        self.related_search_input = related_search_input
        self.related_field = related_field
        
        super().__init__(*args, **kwargs)


    def widget_attrs(self, widget: Widget) -> Any:
        attrs = super().widget_attrs(widget)        
        attrs.update({'related_search_input': self.related_search_input,
                      'related_field': self.related_field,
                      'id': f"char-model-related-field-{random.randint(0, 1000000000)}",
                      })
        return attrs

class CharRelatedField(MainRelatedField, forms.CharField):
    pass

class IntegerRelatedField(MainRelatedField, forms.IntegerField):
    pass

class FloatRelatedField(MainRelatedField, forms.FloatField):
    pass

class DecimalRelatedField(MainRelatedField, forms.DecimalField):
    pass

class DateRelatedField(MainRelatedField, forms.DateField):
    pass
    


from django import forms
from .field import MainRelatedField, SearchModelField, SelectSearchCharField

class RelatedFillForm(forms.Form):
    
    def extract_function_attrs(self, kwargs:dict):
        """
        Extract the function attrs from the kwargs
        """
        function_attrs = {}
        for key, value in kwargs.copy().items():
            if key.endswith('__function_filters'):
                function_attrs[key] = value
                del kwargs[key]
        
        return function_attrs
    
    
    def function_attrs_clean(self, function_attrs:dict):
        """
        Clean the function attrs
        """
        for key, value in function_attrs.copy().items():
            if value is None:
                function_attrs[key] = "null"
            elif value in [True, False]:
                function_attrs[key] = str(value).lower()
        return function_attrs
    
    def __init__(self, *args, **kwargs):
        
        function_attrs = self.extract_function_attrs(kwargs)
        super().__init__(*args, **kwargs)
        
        for field_name, field in self.fields.items():
            # check if the field is class is inherited from MainRelatedField
            if issubclass(field.__class__, MainRelatedField):
                related_search_input_name = field.related_search_input
                related_field = self.fields[related_search_input_name]
                related_field_id = f"search-field-{id(related_field)}"
                self.fields[field_name].widget.attrs['related_search_input_id'] = related_field_id
                
            elif isinstance(field, SearchModelField) or issubclass(field.__class__, SelectSearchCharField):
                field_id = f"search-field-{id(field)}"
                self.fields[field_name].widget.attrs['id'] = field_id
                field_attrs_name = f'{field_name}__function_filters'
                attrs = function_attrs.get(field_attrs_name)
                if not attrs:
                    continue
                attrs = self.function_attrs_clean(attrs)
                for attr_name, attr in attrs.items():
                    attr_name= f'function_filters__{attr_name}'
                    self.fields[field_name].widget.attrs[attr_name] = attr
                




import traceback
from .query_function_register import API_REGISTER
from django.http import JsonResponse
from django.contrib.auth.decorators import login_required
from uuid import uuid4

def get_query_result(request):
    query_name = request.GET.get('query_name')
    query = request.GET.get('q')
    search_field = request.GET.get('search_field')
    
    function_filters = {k.replace('function_filters__', ''): v for k, v in request.GET.items() if k.startswith('function_filters__')}
    function_filters.update({search_field: query})

    for key, value in function_filters.copy().items():
        if not type(value) == str:
            continue
        if value == "null":
            function_filters[key] = None
        elif value.lower() in ["true", "false"]:
            function_filters[key] = value.lower() == "true"
        elif value.isdigit():
            function_filters[key] = int(value)
        elif value.replace('.', '', 1).isdigit():
            function_filters[key] = float(value)

    # get the query function
    query_class = API_REGISTER().get(query_name)
    if not query_class:
        print(f"ERROR: The query function '{query_name}' is not registered.")
        return JsonResponse({"error":""},status=400)
    
    if not query_class().get_permissions(request):
        print(f"ERROR: The user does not have permission to access the query function '{query_name}'.")
        return JsonResponse({"error":""},status=403)
    
    # get the result
    try:
        options = query_class().get_filtered_options(function_filters, search_field)
        result = []
        for option in options:
            result.append(option.to_dict())
        
    except Exception as e:
        uuid = uuid4()
        print(uuid, "ERROR: ", e, traceback.format_exc())
        result = {"error": f"An error occurred. Please contact the administrator. The error id is {uuid}"}
        
    # return the result
    return JsonResponse(result, safe=False)



# fields.py
from typing import Any
from django import forms
from django.forms.widgets import Widget
from ..widgets import SelectSearchTextInput
import random
from ..providers import create_model_provider_from_model
from ..providers import OnlyAuthenticated

class SearchModelField(forms.CharField):
    widget = SelectSearchTextInput

    def __init__(self, model, search_field, permissions=[OnlyAuthenticated], query_function_name='', min_search_length=1,*args, **kwargs):
        if not query_function_name:
            provider = create_model_provider_from_model(model, permissions)
            query_function_name = provider.query_function_name
        self.query_function_name = query_function_name
        self.min_search_length = min_search_length
        self.search_field = search_field
        self.model = model
        super().__init__(*args, **kwargs)

    def widget_attrs(self, widget: Widget) -> Any:
        attrs = super().widget_attrs(widget)        
        attrs.update({'query_function_name': self.query_function_name,
                      'min_search_length': self.min_search_length,
                      'id': f"search-input-{random.randint(0, 1000000000)}",
                      "search_field" : self.search_field,
                      "model_field": True
                      })
        return attrs
    
    def to_python(self, value):
        if not value :
            return None
        return self.model.objects.get(id=value)
    


from .query_function_register import API_REGISTER
from django.db.models import QuerySet
from .options import InputOption, ModelOption

class OnlyAuthenticated:
    def has_permission(self, request):
        return request.user.is_authenticated


class SearchModelProvider:
    model = None
    query_function_name = None
    serializer = None
    auto_register = False

    def object_str(self, obj):
        return str(obj)

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        API_REGISTER().register(cls, cls.query_function_name, cls.auto_register)

    def get_permissions(self, request):
        return request.user.is_authenticated

    def get_queryset(self) -> QuerySet:
        return self.model.objects.all()

    def get_queryset_by_pk(self, pk) -> QuerySet:
        return self.get_queryset().get(pk=pk)

    def get_object_by_pk(self, pk):
        return ModelOption(self.get_queryset_by_pk(pk), self.serializer, self.object_str)

    def get_filtered_queryset(self, function_filters: dict, search_key) -> QuerySet:
        function_filters = {
            key + '__icontains' if key == search_key else key: value 
            for key, value in function_filters.items()
        }
        return self.get_queryset().filter(**function_filters)

    def get_filtered_options(self, function_filters, search_key):
        options = self.get_filtered_queryset(function_filters, search_key)
        return [ModelOption(option, self.serializer, self.object_str) for option in options]

class SearchModelFieldProvider:
    model = None
    query_function_name = None
    auto_register = False

    def object_str(self, obj):
        return str(obj)

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        API_REGISTER().register(cls, cls.query_function_name, cls.auto_register)

    def get_permissions(self, request):
        return request.user.is_authenticated

    def get_queryset(self) -> QuerySet:
        return self.model.objects.all()

    def get_filtered_queryset(self, function_filters: dict, search_key) -> QuerySet:
        function_filters = {
            key + '__icontains' if key == search_key else key: value 
            for key, value in function_filters.items()
        }
        return self.get_queryset().filter(**function_filters).values_list(search_key, flat=True).distinct()

    def get_filtered_options(self, function_filters, search_key):
        options = self.get_filtered_queryset(function_filters, search_key)
        return [InputOption(option, option) for option in options]

class SearchDataProvider:
    json = None
    query_function_name = None
    auto_register = False

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        API_REGISTER().register(cls, cls.query_function_name, cls.auto_register)

    def get_permissions(self, request):
        return request.user.is_authenticated

    def get_filtered_options(self, function_filters, search_key):
        raise NotImplementedError("This method must be implemented in the subclass")

def create_model_provider_from_model(Model, permission_classes=[]):
    class Provider(SearchModelProvider):
        model = Model
        query_function_name = Model.__name__ + '_search'
        auto_register = True
        
        def get_permissions(self, request):
            for permission in permission_classes:
                if not permission().has_permission(request):
                    return False
            return True 
        
    return Provider

def create_model_field_provider_from_model(Model, permission_classes=[]):
    class Provider(SearchModelFieldProvider):
        model = Model
        query_function_name = Model.__name__ + '_search_field'
        auto_register = True

        def get_permissions(self, request):
            for permission in permission_classes:
                if not permission().has_permission(request):
                    return False
            return True
        
    return Provider



import inspect


class API_REGISTER():
    api_classes = {}
    
    
    def __new__(cls):
        if not hasattr(cls, 'instance'):
            cls.instance = super(API_REGISTER, cls).__new__(cls)
        return cls.instance
    
    
    # each function should only take one parameter, which is the query
    def register(self, function, name, auto_register=False):
        assert callable(function), f"The function '{name}' is not callable."
        assert name is not None, f"The function name cannot be None."
        if name in self.api_classes and self.api_classes[name].__module__ == function.__module__:
            return 
        
        elif name in self.api_classes and not auto_register:
            raise ValueError(f"The function name '{name}' is already registered.")
        
        self.api_classes.setdefault(name, function)
    
    def get(self, name):
        return self.api_classes.get(name)

def register_api_function(name):
    def decorator(function):
        API_REGISTER().register(function, name)
        return function
    return decorator

from django.utils.module_loading import autodiscover_modules
autodiscover_modules('search_field_functions')


from .model_search_field import SearchModelField
from .search_field import SelectSearchCharField
from .related_field import MainRelatedField, CharRelatedField, IntegerRelatedField, FloatRelatedField, DecimalRelatedField, DateRelatedField



import json
import json
from django.core.serializers.json import DjangoJSONEncoder
from django.core import serializers


class InputOption:
    def __init__(self, id, string):
        self.id = id
        self.string = string

    def to_dict(self):
        return {
            'id': self.id,
            'text': self.string
        }

class DictOption:
    def __init__(self, id, string, json_data):
        self.id = id
        self.string = string
        self.json = json_data

    def to_dict(self):
        return {
            'id': self.id,
            'text': self.string,
            'fields_data': self.json
        }

class ModelOption:
    def __init__(self, item, serializer=None, str_function=None):
        self.item = item
        self.id = item.pk
        self.string = str_function(item) if str_function else str(item)
        self.json = serializer(item).data if serializer else json.loads(serializers.serialize('json', [item]))[0]['fields']

    def to_dict(self):
        return {
            'id': self.id,
            'text': self.string,
            'fields_data': self.json
        }



from django import template
from ..query_function_register import API_REGISTER

register = template.Library()

@register.simple_tag(name='search_field_get_init_value')
def get_init_selected(function_name,is_model_field, value):
    if not value:
        return None
    if is_model_field:
        option =  API_REGISTER().get(function_name)().get_object_by_pk(value)
        return {'id': option.id, 'string': option.string}
    return {'id': value, 'string': value}
    


from setuptools import setup, find_packages
from pathlib import Path

# Read the README file for the long description
long_description = (Path(__file__).parent / "README.md").read_text()

setup(
    name='django-search-input-field',
    version='0.1.12',
    license='MIT',
    author="Mina Atef",
    author_email='mina.atef0@gmail.com',
    packages=find_packages(include=['django_search_input_field', 'django_search_input_field.*']),
    include_package_data=True,
    package_data={'django_search_input_field': ['templates/*']},
    url='https://github.com/minaaaatef/django-search-input-form',
    description='A Django app providing a search input field.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=[
        # Add any package dependencies here
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Framework :: Django :: 3.2',
    ],
    keywords='django search input field',
    python_requires='>=3.6',
)
